/** @file gui_keybinddata.cpp

	Skelton for retropc emulator

	@author Sasaji
	@date   2015.5.12

	@brief [ keybind data for gui ]
*/

#include "gui_keybinddata.h"
#include "../emu.h"
#include "../clocale.h"
#include "../keycode.h"

KeybindData::KeybindData()
{
	tab_num = 0;
}

KeybindData::~KeybindData()
{
}

void KeybindData::Init(EMU *emu, int new_tabnum)
{
	tab_num = new_tabnum;

	// get parameters
	memset(table, 0, sizeof(table));
	memset(row2idx_map, 0, sizeof(row2idx_map));
	vmkey_map_size = 0;

	vkkey_defmap = NULL;
	vkkey_defmap_rows = 0;
	vkkey_defmap_cols = 0;

	vkkey_map = NULL;
	memset(vkkey_preset_map, 0, sizeof(vkkey_preset_map));

	// set initial data
	int presets = emu->get_parami(VM::ParamVkKeyPresets);
	presets = (presets < KEYBIND_PRESETS ? presets : KEYBIND_PRESETS);

	SetVmKeyMap((uint16 *)emu->get_paramv(VM::ParamVmKeyMap0 + tab_num), emu->get_parami(VM::ParamVmKeyMapSize0 + tab_num));

	SetVkKeyDefMap((uint32 *)emu->get_paramv(VM::ParamVkKeyDefMap0 + tab_num), emu->get_parami(VM::ParamVkKeyMapKeys0 + tab_num), emu->get_parami(VM::ParamVkKeyMapAssign));

	SetVkKeyMap((uint32 *)emu->get_paramv(VM::ParamVkKeyMap0 + tab_num));

	for(int j=0; j<presets; j++) {
		SetVkKeyPresetMap((uint32 *)emu->get_paramv(VM::ParamVkKeyPresetMap00 + tab_num + j * KEYBIND_MAX_NUM), j);
	}

	// set disable key
	max_rows = 0;
	for(int idx=0; idx<KBCTRL_MAX_LINES; idx++) {
		if (table[idx].vm_keycode >= vkkey_defmap_rows) {
			table[idx].enabled = false;
		}

		if (table[idx].enabled) {
			table[idx].row = max_rows;
			row2idx_map[max_rows] = idx;
			max_rows++;
		} else {
			table[idx].row = -1;
		}

		for(int col=0; col<vkkey_defmap_cols; col++) {
			table[idx].cols[col].col = col;
			uint32 code = *(vkkey_map + table[idx].vm_keycode * vkkey_defmap_cols + col);
			if (tab_num == 1) SetVkJoyCode(&table[idx].cols[col],code,NULL);
			else SetVkKeyCode(&table[idx].cols[col],code,NULL);
		}
	}
}

void KeybindData::SetVmKeyMap(uint16 *vmKeyMap, int size)
{
	if (tab_num < 0 || tab_num >= KEYBIND_MAX_NUM) return;

	for(int i=0; i<KBCTRL_MAX_LINES && i<size; i++) {
		SetVmKeyCode(i,vmKeyMap[i]);
	}
	vmkey_map_size = (size > KBCTRL_MAX_LINES) ? KBCTRL_MAX_LINES : size;
}

void KeybindData::SetVmKey(int idx, uint16 code)
{
	if (tab_num < 0 || tab_num >= KEYBIND_MAX_NUM) return;

	//	vmkey_map[idx] = code;
	SetVmKeyCode(idx,code);
}

bool KeybindData::SetVmKeyCode(int idx, uint16 code)
{
	if (idx < 0 || idx >= KBCTRL_MAX_LINES) {
		return false;
	}
	if (code >= 0xff) {
		table[idx].enabled = false;
	} else {
		table[idx].enabled = true;
	}

	table[idx].vm_keycode = code;

	return true;
}

void KeybindData::SetVkKeyMap(uint32 *vkKeyMap)
{
	if (tab_num < 0 || tab_num >= KEYBIND_MAX_NUM) return;

	vkkey_map = vkKeyMap;
}

void KeybindData::SetVkKeyDefMap(uint32 *vkKeyDefMap, int rows, int cols)
{
	if (tab_num < 0 || tab_num >= KEYBIND_MAX_NUM) return;

	vkkey_defmap = vkKeyDefMap;
	vkkey_defmap_rows = rows;
	vkkey_defmap_cols = cols;
}

void KeybindData::SetVkKeyPresetMap(uint32 *vkKeyMap, int idx)
{
	if (tab_num < 0 || tab_num >= KEYBIND_MAX_NUM) return;

	if (idx < 4) {
		vkkey_preset_map[idx] = vkKeyMap;
	}
}

bool KeybindData::SetVkKeyCode(codecols_t *obj, uint32 code, _TCHAR *label)
{
	if (label != NULL) GetVkKeyLabel(code,label,true);
	obj->vk_prev_keycode = obj->vk_keycode;
	obj->vk_keycode = code;

	return true;
}

bool KeybindData::SetVkKeyCode(int row, int col, uint32 code, _TCHAR *label)
{
	int idx = row2idx_map[row];
	codecols_t *obj = &table[idx].cols[col];
	return SetVkKeyCode(obj,code,label);
}

bool KeybindData::SetVkJoyCode(codecols_t *obj, uint32 code, _TCHAR *label)
{
	bool rc = true;
	if (label != NULL) {
		rc = GetVkJoyLabel(code,label,true);
	}
	if (rc == true) {
		obj->vk_prev_keycode = obj->vk_keycode;
		obj->vk_keycode = code;
	}
	return true;
}

bool KeybindData::SetVkJoyCode(int row, int col, uint32 code, _TCHAR *label)
{
	int idx = row2idx_map[row];
	codecols_t *obj = &table[idx].cols[col];
	return SetVkJoyCode(obj,code,label);
}

bool KeybindData::SetVkCode(int row, int col, uint32 code, _TCHAR *label)
{
	if (tab_num == 1) {
		return SetVkJoyCode(row, col, code, label);
	} else {
		return SetVkKeyCode(row, col, code, label);
	}
}

uint32 KeybindData::GetCombi()
{
	return *(vkkey_map + (vkkey_defmap_rows - 1) * vkkey_defmap_cols);
}
void KeybindData::SetCombi(uint32 value)
{
	*(vkkey_map + (vkkey_defmap_rows - 1) * vkkey_defmap_cols) = value;
}

const _TCHAR *KeybindData::GetCellString(int row, int col)
{
	static _TCHAR label[128];
	GetCellString(row, col, label);
	return label;
}

bool KeybindData::GetCellString(int row, int col, _TCHAR *label)
{
	bool rc = false;
	int idx = row2idx_map[row];
	if(col < 0) {
		GetVmKeyLabel(idx, label, true);
	} else {
		if (tab_num == 1) rc = GetVkJoyLabel(table[idx].cols[col].vk_keycode,label,true);
		else rc = GetVkKeyLabel(table[idx].cols[col].vk_keycode,label,true);
	}
	return rc;
}

bool KeybindData::IsEnable(int index)
{
	return table[index].enabled;
}

void KeybindData::LoadDefaultPreset()
{
	LoadPreset(-1);
}

void KeybindData::LoadPreset(int idx)
{
	int rows = vmkey_map_size;
	int cols = vkkey_defmap_cols;
	uint32 *dst;
	uint32 code;

	if (0 <= idx && idx <= KEYBIND_PRESETS) {
		dst = vkkey_preset_map[idx];
	} else {
		dst = vkkey_defmap;
	}

	for(int row=0; row<rows; row++) {
		if (table[row].enabled) {
			for(int col=0; col<cols; col++) {
				code = dst[table[row].vm_keycode * cols + col];
				if (tab_num == 1) SetVkJoyCode(&table[row].cols[col],code,NULL);
				else SetVkKeyCode(&table[row].cols[col],code,NULL);
			}
		}
	}
	if (tab_num == 1) {
		SetCombi(dst[(vkkey_defmap_rows - 1) * cols]);
	}
}

void KeybindData::SavePreset(int idx)
{
	int rows = vmkey_map_size;
	int cols = vkkey_defmap_cols;
	uint32 *dst;
	uint32 code;

	if (0 <= idx && idx <= KEYBIND_PRESETS) {
		dst = vkkey_preset_map[idx];
	} else {
		return;
	}

	for(int row=0; row<rows; row++) {
		if (table[row].enabled) {
			for(int col=0; col<cols; col++) {
				code = table[row].cols[col].vk_keycode;
				dst[table[row].vm_keycode * cols + col] = code;
			}
		}
	}
	if (tab_num == 1) {
		dst[(vkkey_defmap_rows - 1) * cols] = GetCombi();
	}
}

void KeybindData::SetData()
{
	int rows = vmkey_map_size;
	int cols = vkkey_defmap_cols;
	uint32 *dst = vkkey_map;
	uint32 code = 0;

	for(int row=0; row<rows; row++) {
		if (table[row].enabled) {
			for(int col=0; col<cols; col++) {
				code = table[row].cols[col].vk_keycode;
				dst[table[row].vm_keycode * cols + col] = code;
			}
		}
	}
}

bool KeybindData::GetVmKeyLabel(int code, _TCHAR *label, bool translate) {
	bool known_key = true;
	const _TCHAR *p = NULL;
	const _TCHAR *fp = NULL;

	switch(code) {
		case 0x00:
			p = _T("");
			break;
		case 0x01:
			p = _TX("BREAK");
			break;
		case 0x02:
			p = _TX("RESET");
			break;
		case 0x03:
			p = _TX("MODE");
			break;
		case 0x04:
			p = _TX("(Pause)");
			break;
		case 0x05:
			p = _TX("(Alt)");
			break;
		case 0x06:
			p = _TX("INS/DEL");
			break;
		case 0x07:
			p = _TX("num INS/DEL");
			break;
		case 0x08:
			p = _TX("BS");
			break;
		case 0x09:
			p = _TX("TAB");
			break;
		case 0x0c:
			p = _TX("HOME/CLS");
			break;
		case 0x0d:
			p = _TX("RETURN");
			break;
		case 0x0e:
			p = _TX("ENTER");
			break;
		case 0x0f:
			p = _TX("HELP");
			break;

		case 0x10:
			p = _TX("SHIFT");
			break;
		case 0x11:
			p = _TX("CTRL");
			break;
		case 0x14:
			p = _TX("CAPS LOCK");
			break;
		case 0x15:
			p = _TX("KATA/HIRA");
			break;
		case 0x16:
			p = _TX("GRAPH");
			break;
		case 0x17:
			p = _TX("HENKAN");
			break;
		case 0x18:
			p = _TX("MUHENKAN");
			break;
		case 0x19:
			p = _TX("COMMAND");
			break;
		case 0x1a:
			p = _TX("OPTION");
			break;

		case 0x1b:
			p = _TX("ESC");
			break;
		case 0x1c:
			p = _TX("RIGHT");
			break;
		case 0x1d:
			p = _TX("LEFT");
			break;
		case 0x1e:
			p = _TX("UP");
			break;
		case 0x1f:
			p = _TX("DOWN");
			break;
		case 0x20:
			p = _TX("SPACE");
			break;
		// 0x21 - 0x28  button1 - 8

		// 0x29 - 0x2f  unknown

		// 0x30 - 0x39  0 - 9
		case 0x3a:
			p = _TX(": *");
			break;
		case 0x3b:
			p = _TX("; +");
			break;
		case 0x3c:
			p = _TX(", <");
			break;
		case 0x3d:
			p = _TX("- =");
			break;
		case 0x3e:
			p = _TX(". >");
			break;
		case 0x3f:
			p = _TX("/ ?");
			break;
		case 0x40:
			p = _TX("@");
			break;
		// 0x41 - 0x5a  A - Z
		case 0x5b:
			p = _TX("[");
			break;
		case 0x5c:
			p = _TX("\\");
			break;
		case 0x5d:
			p = _TX("]");
			break;
		case 0x5e:
			p = _TX("^");
			break;
		case 0x5f:
			p = _TX("_");
			break;
		// 0x60 - 0x69  num 0 - num 9
		case 0x6a:
			p = _TX("num *");
			break;
		case 0x6b:
			p = _TX("num +");
			break;
		case 0x6c:
			p = _TX("num ?");
			break;
		case 0x6d:
			p = _TX("num -");
			break;
		case 0x6e:
			p = _TX("num .");
			break;
		case 0x6f:
			p = _TX("num /");
			break;
		// 0x70 - 0x7f  PF1 - PF16

		default:
			if (0x30 <= code && code <= 0x39) {
				// 0 - 9
				_stprintf(label, _T("%c") ,code);
			} else if (0x41 <= code && code <= 0x5a) {
				// A - Z
				_stprintf(label, _T("%c"), code);
			} else if (0x60 <= code && code <= 0x69) {
				// num 0 - num 9
				fp = _TX("num %c");
				if (translate) fp = _tgettext(fp);
				_stprintf(label,fp,code - 0x30);
			} else if (0x70 <= code && code <= 0x7f) {
				// PF1 - PF16
				fp = _TX("PF%d");
				if (translate) fp = _tgettext(fp);
				_stprintf(label,fp,code - 0x6f);
			} else {
				_stprintf(label, _T("0x%02x") ,code);
				known_key = false;
			}
			break;
	}
	if (p != NULL) {
		if (translate && *p != 0) p = _tgettext(p);
		_tcscpy(label, p);
	}

	return known_key;
}

bool KeybindData::GetVkKeyLabel(uint32 code, _TCHAR *label, bool translate) {
	bool known_key = true;
	const _TCHAR *p = NULL;
	const _TCHAR *fp = NULL;

	switch(code) {
		case 0:
			p = _T("");
			break;
		case VK_BACK:
			p = _TX("bs");
			break;
		case VK_TAB:
			p = _TX("tab");
			break;
		case VK_CLEAR:
			p = _TX("clear");
			break;
		case VK_RETURN:
			p = _TX("enter");
			break;
		case VK_PAUSE:
			p = _TX("pause");
			break;

		case VK_ESCAPE:
			p = _TX("esc");
			break;
		case VK_SPACE:
			p = _TX("space");
			break;

		case VK_OEM_COMMA:
			p = _TX(", <");	// JP
			break;
		case VK_OEM_MINUS:
			p = _TX("- =");	// JP
			break;
		case VK_OEM_PERIOD:
			p = _TX(". >");	// JP
			break;
		case VK_OEM_2:
			p = _TX("/ ?");	// JP
			break;

		case VK_OEM_1:
			p = _TX(": *");	// JP
			break;
		case VK_OEM_PLUS:
			p = _TX("; +");	// JP
			break;
		case VK_OEM_3:
			p = _TX("@");	// JP
			break;
		case VK_OEM_4:
			p = _TX("[");	// JP
			break;
		case VK_OEM_5:
			p = _TX("\\");	// JP
			break;
		case VK_OEM_6:
			p = _TX("]");	// JP
			break;
		case VK_OEM_7:
			p = _TX("^");	// JP
			break;
		case VK_OEM_102:
			p = _TX("_");	// JP
			break;
		case VK_KANJI:
			p = _TX("kanji");	// JP
			break;

		case VK_DELETE:
			p = _TX("delete");
			break;

		case VK_CONVERT:
			p = _TX("henkan");
			break;
		case VK_NONCONVERT:
			p = _TX("muhenkan");
			break;
		case VK_KANA:
			p = _TX("katakana");
			break;

		case VK_DECIMAL:
			p = _TX("num .");
			break;
		case VK_DIVIDE:
			p = _TX("num /");
			break;
		case VK_MULTIPLY:
			p = _TX("num *");
			break;
		case VK_SUBTRACT:
			p = _TX("num -");
			break;
		case VK_ADD:
			p = _TX("num +");
			break;
		case VK_NUMLOCK:
			p = _TX("numlock");
			break;
//		case KEYCODE_KP_ENTER:
//			p = _TX("num enter");
//			break;
//		case KEYCODE_KP_EQUALS:
//			p = _TX("num =");
//			break;

		case VK_UP:
			p = _TX("up");
			break;
		case VK_DOWN:
			p = _TX("down");
			break;
		case VK_RIGHT:
			p = _TX("right");
			break;
		case VK_LEFT:
			p = _TX("left");
			break;
		case VK_INSERT:
			p = _TX("insert");
			break;
		case VK_HOME:
			p = _TX("home");
			break;
		case VK_END:
			p = _TX("end");
			break;
		case VK_PRIOR:
			p = _TX("page up");
			break;
		case VK_NEXT:
			p = _TX("page down");
			break;

//		case KEYCODE_NUMLOCKCLEAR:
//			p = _TX("numlock");
//			break;
		case VK_CAPITAL:
			p = _TX("caps");
			break;
		case VK_SCROLL:
			p = _TX("scroll lock");
			break;
		case VK_SHIFT:
			p = _TX("shift");
			break;
//		case KEYCODE_LSHIFT:
//			p = _TX("lshift");
//			break;
		case VK_CONTROL:
			p = _TX("ctrl");
			break;
//		case KEYCODE_LCTRL:
//			p = _TX("lctrl");
//			break;
		case VK_MENU:
			p = _TX("alt");
			break;
//		case KEYCODE_LALT:
//			p = _TX("lalt");
//			break;
		case VK_RWIN:
			p = _TX("right win");
			break;
		case VK_LWIN:
			p = _TX("left win");
			break;
//		case KEYCODE_MODE:
//			p = _TX("mode");
//			break;
		case VK_HELP:
			p = _TX("help");
			break;
//		case KEYCODE_SYSREQ:
//			p = _TX("sys req");
//			break;
		case VK_APPS:
			p = _TX("menu");
			break;
#if 0
		case KEYCODE_RSUPER:
			p = _TX("rsuper");
			break;
		case KEYCODE_LSUPER:
			p = _TX("lsuper");
			break;
		case KEYCODE_COMPOSE:
			p = _TX("compose");
			break;
		case KEYCODE_PRINT:
			p = _TX("print");
			break;
		case KEYCODE_BREAK:
			p = _TX("break");
			break;
#endif
		case '0':
			p = _T("0");
			break;
		case VK_NUMPAD0:
			p = _TX("num 0");
			break;
		default:
			if ('1' <= code && code <= '9') {
				// 1 - 9
				_stprintf(label,_T("%d"),code + 1 - '1');
			} else if ('A' <= code && code <= 'Z') {
				// A - Z
				_stprintf(label,_T("%c"),code);
			} else if (VK_NUMPAD1 <= code && code <= VK_NUMPAD9) {
				// num 1 - num 9
				fp = _TX("num %d");
				if (translate) fp = _tgettext(fp);
				_stprintf(label,fp,code + 1 - VK_NUMPAD1);
			} else if (VK_F1 <= code && code <= VK_F12) {
				// F1 - F12
				fp = _TX("F%d");
				if (translate) fp = _tgettext(fp);
				_stprintf(label,fp,code + 1 - VK_F1);
			} else if (VK_F13 <= code && code <= VK_F24) {
				// F13 - F24
				fp = _TX("F%d");
				if (translate) fp = _tgettext(fp);
				_stprintf(label,fp,code + 13 - VK_F13);
			} else {
				_stprintf(label,_T("0x%02x"),code);
				known_key = false;
			}
			break;
	}
	if (p != NULL) {
		if (translate && *p != 0) p = _tgettext(p);
		_tcscpy(label, p);
	}

	return known_key;
}

bool KeybindData::GetVkJoyLabel(uint32 code, _TCHAR *label, bool translate) {
	bool known_key = true;
	_TCHAR numstr[8];
	const _TCHAR *p = NULL;
	const _TCHAR *fp = NULL;

	switch(code) {
		case 0x00:
			p = _T("");
			break;
		case 0x01:
			p = _TX("up");
			break;
		case 0x02:
			p = _TX("down");
			break;
		case 0x04:
			p = _TX("left");
			break;
		case 0x05:
			p = _TX("up+left");
			break;
		case 0x06:
			p = _TX("down+left");
			break;
		case 0x08:
			p = _TX("right");
			break;
		case 0x09:
			p = _TX("up+right");
			break;
		case 0x0a:
			p = _TX("down+right");
			break;
		default:
			// button
			if (code & 0xfffffff0) {
				fp = _TX("button");
				if (translate) fp = _tgettext(fp);
				_tcscpy(label, fp);
				for(int i=0; i<28; i++) {
					if (code & (0x10 << i)) {
						_stprintf(numstr, _T(" %d"), i+1);
						strcat(label, numstr);
					}
				}
			} else {
				_tcscpy(label, _T(""));
				known_key = false;
			}
			break;
	}
	if (p != NULL) {
		if (translate && *p != 0) p = _tgettext(p);
		_tcscpy(label, p);
	}

	return known_key;
}
