/** @file win_gui.h

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2014.03.24 -

	@brief [ gui for windows ]
*/

#ifndef _WIN_GUI_H_
#define _WIN_GUI_H_

#include <Windows.h>
#include "../gui_base.h"
#include "../../common.h"
#include "../../winfont.h"
#include "win_configbox.h"
#include "win_keybindbox.h"
#include "win_volumebox.h"
#include "win_recvidbox.h"
#include "win_aboutbox.h"
#ifdef USE_DEBUG_SOUND_FILTER
#include "win_sndfilterbox.h"
#endif

namespace GUI_WIN
{

bool start_auto_key_from_clipboard(EMU *emu);

#ifdef GUI_TYPE_WINDOWS

class GUI : public GUI_BASE
{
private:
	HINSTANCE hInstance;
	CFont *font;
	ConfigBox *configbox;
	KeybindBox *keybindbox;
	VolumeBox *volumebox;
	RecVideoBox *recvidbox;
	AboutBox *aboutbox;
#ifdef USE_DEBUG_SOUND_FILTER
	SndFilterBox *sndfilterbox;
#endif

	/// menu
	bool now_menu;
	bool now_menuloop;

public:
	GUI(int argc, char **argv, EMU *new_emu);
	virtual ~GUI();

	virtual int CreateWidget(HWND new_hwnd, int width, int height);

	virtual int CreateMenu();

	virtual void ShowMenu();
	virtual void HideMenu();

	virtual LRESULT ProcessEvent(UINT iMsg, WPARAM wParam, LPARAM lParam);
	virtual void ScreenModeChanged(bool fullscreen);

	void SetWindowHandle(HWND new_hwnd) { hWindow = new_hwnd; }

	virtual void SetFocusToMainWindow();

	virtual bool ShowRecordVideoDialog(int num);

	virtual bool ShowAboutDialog();
#ifdef USE_EMU_INHERENT_SPEC
	virtual bool ShowConfigureDialog();
	virtual bool ShowKeybindDialog();
	virtual bool ShowVolumeDialog();
	virtual bool ShowSndFilterDialog();
#endif
#ifdef USE_DATAREC
	virtual bool ShowLoadDataRecDialog();
	virtual bool ShowSaveDataRecDialog();
	virtual void set_datarec_file_menu(HMENU hMenu, UINT uItem);
#endif

#ifdef USE_FD1
	virtual bool ShowOpenFloppyDiskDialog(int drv);
	void set_disk_file_menu(HMENU hMenu, UINT uItem, int drv);
#ifdef USE_EMU_INHERENT_SPEC
	void set_disk_side_menu(HMENU hMenu, UINT uItem, int drv);
#endif
#endif

#ifdef USE_AUTO_KEY
	virtual bool ShowOpenAutoKeyDialog();
	virtual bool StartAutoKey(void);
#endif

#ifdef USE_EMU_INHERENT_SPEC
	virtual bool ShowSaveStateDialog(bool cont);
	virtual bool ShowLoadStateDialog();
#ifdef USE_KEY_RECORD
	virtual bool ShowPlayRecKeyDialog();
	virtual bool ShowRecordRecKeyDialog();
	virtual bool ShowRecordStateAndRecKeyDialog();
#endif
#endif

#ifdef USE_PRINTER
	virtual bool ShowSavePrinterDialog(int drv);
#endif

#ifdef USE_CART
	void open_cart_dialog(HWND hWnd);
#endif
#ifdef USE_QUICKDISK
	void open_quickdisk_dialog(HWND hWnd);
#endif
#ifdef USE_MEDIA
	void open_media_dialog(HWND hWnd);
#endif
#ifdef USE_BINARY_FILE1
	void open_binary_dialog(HWND hWnd, int drv, bool load);
#endif

	virtual bool OpenDroppedFile(void *param);

	void update_menu(HMENU hMenu, int pos);
	void translate_menu(HMENU hMenu, int depth);
};

#endif /* GUI_TYPE_WINDOWS */

}; /* namespace GUI_WIN */

#endif /* _WIN_GUI_H_ */
