/** @file ledbox.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2011.08.08 -

	@brief [ led box ]
*/
#include "ledbox.h"
#include "emu.h"
#include <windowsx.h>

#define RECT_IN(st, xx, yy, ww, hh) { \
	st.pt.x = xx; \
	st.pt.y = yy; \
	st.sz.cx = ww; \
	st.sz.cy = hh; \
}

#define POINT_IN(st, xx, yy, sf) { \
	st.pt.x = xx; \
	st.pt.y = yy; \
}

LedBox::LedBox(HINSTANCE hInst) : DibSection()
{
	hInstance = hInst;
	hLedBox = NULL;

	for(int i=0; i<LED_TYPE_END; i++) {
		ZeroMemory(&led[i], sizeof(led_t));
		led[i].hBmp = NULL;
		led[i].hDC = NULL;
	}
	ZeroMemory(led_pt, sizeof(led_pt));

	flag_old = -1;
	dist.x = 0; dist.y = 0;
	mode = 0;
//	dist_def[0].x = 2; dist_def[0].y = 2;
//	dist_def[1].x = 2; dist_def[1].y = 2;
	dist_set[0].x = 2; dist_set[0].y = 2;
	dist_set[1].x = 2; dist_set[1].y = 2;
	memset(&win_pt, 0, sizeof(win_pt));

	visible = false;
	inside = false;
}

LedBox::~LedBox()
{
	for(int i=0; i<LED_TYPE_END; i++) {
		if (led[i].hDC) DeleteDC(led[i].hDC);
		if (led[i].hBmp) DeleteObject(led[i].hBmp);
	}
}

// ʏ
bool LedBox::InitScreen(HDC hdc, int width, int height)
{
	enable = false;

	// LEDwi
	if (!create_bitmap(hdc, &led[LED_TYPE_BASE], IDB_KBLEDBOX)) return enable;

	win_sz.cx = led[LED_TYPE_BASE].sz.cx;
	win_sz.cy = led[LED_TYPE_BASE].sz.cy;

	// Dib쐬
	if (!CreateDib(hdc, win_sz.cx, win_sz.cy)) return enable;

	// LEDp[c쐬
	if (!create_bitmap(hdc, &led[LED_TYPE_PARTS], IDB_LED_PARTS)) return enable;

	// horiz
	if (!create_parts(hdc, led[LED_TYPE_PARTS], 7 * 16, 0, 13, 6, &led[LED_TYPE_HR])) return enable;
	if (!create_parts(hdc, led[LED_TYPE_PARTS], 9 * 16, 0, 13, 6, &led[LED_TYPE_HG])) return enable;
	// vert
	if (!create_parts(hdc, led[LED_TYPE_PARTS], 1 * 16, 0, 6, 13, &led[LED_TYPE_VR])) return enable;
	if (!create_parts(hdc, led[LED_TYPE_PARTS], 3 * 16, 0, 6, 13, &led[LED_TYPE_VG])) return enable;
	// datarec
	if (!create_parts(hdc, led[LED_TYPE_PARTS], 11 * 16, 0, 13, 11, &led[LED_TYPE_RR])) return enable;	// play led
	if (!create_parts(hdc, led[LED_TYPE_PARTS], 13 * 16, 0, 11, 11, &led[LED_TYPE_CL])) return enable;	// rec led
	// digit
	if (!create_bitmap(hdc, &led[LED_TYPE_DIGIT], IDB_DIGIT_10)) return enable;
	// tape icon
	if (!create_parts(hdc, led[LED_TYPE_PARTS], 14 * 16, 0, 15, 11, &led[LCD_TYPE_TAPE])) return enable;
	// fdd icon
	if (!create_parts(hdc, led[LED_TYPE_PARTS], 15 * 16, 0, 13, 13, &led[LCD_TYPE_FDD0])) return enable;
	if (!create_parts(hdc, led[LED_TYPE_PARTS], 16 * 16, 0, 13, 13, &led[LCD_TYPE_FDD1])) return enable;
	if (!create_parts(hdc, led[LED_TYPE_PARTS], 17 * 16, 0, 13, 13, &led[LCD_TYPE_FDD2])) return enable;
	if (!create_parts(hdc, led[LED_TYPE_PARTS], 18 * 16, 0, 13, 13, &led[LCD_TYPE_FDD3])) return enable;
	if (!create_parts(hdc, led[LED_TYPE_PARTS], 19 * 16, 0, 8, 15, &led[LCD_TYPE_3_FDD35])) return enable;
	if (!create_parts(hdc, led[LED_TYPE_PARTS], 20 * 16, 0, 8, 15, &led[LCD_TYPE_5_FDD35])) return enable;

	// ʒuݒ
	POINT_IN(led_pt[LED_POS_BASE], 0, 0, led[LED_TYPE_BASE]);
	// keyboard led
	POINT_IN(led_pt[LED_POS_KATAKANA], 4, 3, led[LED_TYPE_HR]);
	POINT_IN(led_pt[LED_POS_HIRAGANA], 4, 11, led[LED_TYPE_HG]);
	POINT_IN(led_pt[LED_POS_CAPS], 36, 3, led[LED_TYPE_VR]);
	// datarec led
	POINT_IN(led_pt[LED_POS_PLAY], 65, 4, led[LED_TYPE_RR]);
	POINT_IN(led_pt[LCD_POS_PLAY], 78, 4, led[LCD_TYPE_TAPE]);
	POINT_IN(led_pt[LED_POS_REC],  96, 4, led[LED_TYPE_CL]);
	POINT_IN(led_pt[LCD_POS_REC], 108, 4, led[LCD_TYPE_TAPE]);
	// fdd type
	POINT_IN(led_pt[LCD_POS_FDD35], 162, 2, led[LCD_TYPE_3_FDD35]);
	// fdd 0
	POINT_IN(led_pt[LED_POS_FDD0], 173, 3, led[LED_TYPE_VR]);
	POINT_IN(led_pt[LCD_POS_FDD0], 180, 3, led[LCD_TYPE_FDD0]);
	// fdd 1
	POINT_IN(led_pt[LED_POS_FDD1], 198, 3, led[LED_TYPE_VR]);
	POINT_IN(led_pt[LCD_POS_FDD1], 205, 3, led[LCD_TYPE_FDD1]);
	// fdd 2
	POINT_IN(led_pt[LED_POS_FDD2], 223, 3, led[LED_TYPE_VR]);
	POINT_IN(led_pt[LCD_POS_FDD2], 230, 3, led[LCD_TYPE_FDD2]);
	// fdd 3
	POINT_IN(led_pt[LED_POS_FDD3], 248, 3, led[LED_TYPE_VR]);
	POINT_IN(led_pt[LCD_POS_FDD3], 255, 3, led[LCD_TYPE_FDD3]);
	// digit
	RECT_IN(led_pt[LED_POS_DIGIT1], 147, 4, 8, 12);	// 1
	RECT_IN(led_pt[LED_POS_DIGIT2], 138, 4, 8, 12);	// 10
	RECT_IN(led_pt[LED_POS_DIGIT3], 129, 4, 8, 12);	// 100


	//
	enable = true;
	copy_led(led_pt[LED_POS_BASE], led[LED_TYPE_BASE]);
	return enable;
}

bool LedBox::create_bitmap(HDC hdc, led_t *l, DWORD id)
{
	BITMAPINFO bInfo;
	ZeroMemory(&bInfo, sizeof(bInfo));
	bInfo.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);

	l->hBmp = LoadBitmap(hInstance,MAKEINTRESOURCE(id));
	if (l->hBmp == NULL) return false;
	l->hDC = CreateCompatibleDC(hdc);
	if (l->hDC == NULL) return false;
	SelectObject(l->hDC, l->hBmp);
	GetDIBits(hdc, l->hBmp, 0, 1, NULL, &bInfo, DIB_RGB_COLORS);
	l->sz.cx = bInfo.bmiHeader.biWidth;
	l->sz.cy = bInfo.bmiHeader.biHeight;

	return true;
}

bool LedBox::create_parts(HDC hdc, const led_t &srcl, int x, int y, int w, int h, led_t *l)
{
	BITMAPINFO bInfo;
	ZeroMemory(&bInfo, sizeof(bInfo));
	bInfo.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	GetDIBits(hdc, srcl.hBmp, 0, 1, NULL, &bInfo, DIB_RGB_COLORS);
	bInfo.bmiHeader.biWidth = w;
	bInfo.bmiHeader.biHeight = h;

	l->hBmp = CreateDIBitmap(hdc, &bInfo.bmiHeader, 0, NULL, NULL, DIB_RGB_COLORS);
	if (l->hBmp == NULL) return false;
	l->hDC = CreateCompatibleDC(hdc);
	if (l->hDC == NULL) return false;
	SelectObject(l->hDC, l->hBmp);
	l->sz.cx = bInfo.bmiHeader.biWidth;
	l->sz.cy = bInfo.bmiHeader.biHeight;

	BitBlt(l->hDC, 0, 0, l->sz.cx, l->sz.cy, srcl.hDC, x, y, SRCCOPY);

	return true;
}

void LedBox::Create(HWND hWnd)
{
	if (!enable) return;

	hLedBox = CreateDialogParam(hInstance, MAKEINTRESOURCE(IDD_KBLEDBOX), hWnd, LedBoxProc, (LPARAM)this);
}

void LedBox::Show(int flag)
{
	visible = ((flag & 1) != 0);
	inside = ((flag & 8) != 0);

	if (hLedBox) {
		ShowWindow(hLedBox, (visible && !inside) ? SW_SHOWNORMAL : SW_HIDE);
		HWND hWnd = GetParent(hLedBox);
		SetActiveWindow(hWnd);
	}
}

void LedBox::Move()
{
	WINDOWINFO wi;
	WINDOWINFO wid;

	if (hLedBox) {
		HWND hWnd = GetParent(hLedBox);
		GetWindowInfo(hWnd, &wi);
		GetWindowInfo(hLedBox, &wid);
		int w = wid.rcWindow.right - wid.rcWindow.left;
		int h = wid.rcWindow.bottom - wid.rcWindow.top;
		int x = wi.rcClient.left + dist.x;
		int y = wi.rcClient.top + dist.y;
		MoveWindow(hLedBox, x, y, w, h, true);
	}
}

void LedBox::MoveInPlace()
{
	WINDOWINFO wi;
	WINDOWINFO wid;

	if (hLedBox) {
		HWND hWnd = GetParent(hLedBox);
		GetWindowInfo(hWnd, &wi);
		GetWindowInfo(hLedBox, &wid);
		int w = wid.rcWindow.right - wid.rcWindow.left;
		int h = wid.rcWindow.bottom - wid.rcWindow.top;
		if (win_pt.place & 1) {
			dist.x = wi.rcClient.right - wi.rcClient.left - w;
		} else {
			dist.x = 0;
		}
		if (win_pt.place & 2) {
			dist.y = wi.rcClient.bottom - wi.rcClient.top - h;
		} else {
			dist.y = 0;
		}
		int x = wi.rcClient.left + dist.x;
		int y = wi.rcClient.top + dist.y;
		MoveWindow(hLedBox, x, y, w, h, true);
	}
}

void LedBox::mouse_move(HWND hDlg, const POINT &pStart, const LPARAM &lParam)
{
	WINDOWINFO wi;
	WINDOWINFO wid;
	HWND hDCtl;

	hDCtl = GetParent(hDlg);
	GetWindowInfo(hDCtl, &wi);
	GetWindowInfo(hDlg, &wid);
	int w = wid.rcWindow.right - wid.rcWindow.left;
	int h = wid.rcWindow.bottom - wid.rcWindow.top;
	int x = GET_X_LPARAM(lParam) - pStart.x + wid.rcWindow.left;
	int y = GET_Y_LPARAM(lParam) - pStart.y + wid.rcWindow.top;
	dist.x = wid.rcWindow.left - wi.rcClient.left;
	dist.y = wid.rcWindow.top - wi.rcClient.top;
	MoveWindow(hDlg, x, y, w, h, true);
}

void LedBox::adjust_dialog_size(HWND hDlg)
{
	WINDOWINFO wi;
	GetWindowInfo(hDlg, &wi);
	int width = led[LED_TYPE_BASE].sz.cx + wi.rcWindow.right - wi.rcWindow.left - wi.rcClient.right + wi.rcClient.left;
	int height = led[LED_TYPE_BASE].sz.cy + wi.rcWindow.bottom - wi.rcWindow.top - wi.rcClient.bottom + wi.rcClient.top;
	SetWindowPos(hDlg, NULL, 0, 0, width, height, SWP_NOMOVE | SWP_NOZORDER);
}

INT_PTR CALLBACK LedBox::LedBoxProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	UNREFERENCED_PARAMETER(lParam);
	HWND hDCtl;
	static POINT pStart;
	LedBox *myObj;

	switch (message)
	{
	case WM_INITDIALOG:
	{
		SetWindowLongPtr(hDlg, DWLP_USER, lParam);
		myObj = (LedBox *)lParam;
		myObj->adjust_dialog_size(hDlg);
		return (INT_PTR)TRUE;
	}
	case WM_LBUTTONDOWN:
		pStart.x = GET_X_LPARAM(lParam);
		pStart.y = GET_Y_LPARAM(lParam);
		return (INT_PTR)FALSE;
	case WM_MOUSEMOVE:
		myObj = (LedBox *)GetWindowLongPtr(hDlg, DWLP_USER);

		if (wParam & MK_LBUTTON)
		{
			myObj->mouse_move(hDlg, pStart, lParam);
		}
		return (INT_PTR)FALSE;
	case WM_LBUTTONUP:
		hDCtl = GetParent(hDlg);
		SetActiveWindow(hDCtl);
		return (INT_PTR)FALSE;
	case WM_COMMAND:
		return (INT_PTR)TRUE;
		break;
	case WM_PAINT:
		{
			myObj = (LedBox *)GetWindowLongPtr(hDlg, DWLP_USER);
			PAINTSTRUCT ps;
			HDC hdc;
			hdc = BeginPaint(hDlg, &ps);
			BitBlt(hdc, 0, 0, myObj->win_sz.cx, myObj->win_sz.cy, myObj->hMainDib, 0, 0, SRCCOPY);
			EndPaint(hDlg, &ps);
			return (INT_PTR)FALSE;
		}
		break;
	}
	return (INT_PTR)FALSE;
}

void LedBox::Update(uint64 flag)
{
	if (!enable) return;

	if (flag != flag_old) {
		copy_led(led_pt[LED_POS_BASE], led[LED_TYPE_BASE]);

		// katakana
		if (flag & 0x0001) {
			copy_led(led_pt[LED_POS_KATAKANA], led[LED_TYPE_HR]);
		}
		// hiragana
		if (flag & 0x0002) {
			copy_led(led_pt[LED_POS_HIRAGANA], led[LED_TYPE_HG]);
		}
		// caps
		if (!(flag & 0x0004)) {
			copy_led(led_pt[LED_POS_CAPS], led[LED_TYPE_VR]);
		}
		// play
		if (flag & 0x0010) {
			copy_led(led_pt[LED_POS_PLAY], led[LED_TYPE_RR]);
		}
		// rec
		if (flag & 0x0020) {
			copy_led(led_pt[LED_POS_REC], led[LED_TYPE_CL]);
		}
		// play lcd icon
		if (flag & 0x0040) {
			copy_led(led_pt[LCD_POS_PLAY], led[LCD_TYPE_TAPE]);
		}
		// rec lcd icon
		if (flag & 0x0080) {
			copy_led(led_pt[LCD_POS_REC], led[LCD_TYPE_TAPE]);
		}

		// fdd type
		if (flag & 0x1000000000) {
			copy_led(led_pt[LCD_POS_FDD35], led[LCD_TYPE_3_FDD35]);
		} else if (flag & 0x2000000000) {
			copy_led(led_pt[LCD_POS_FDD35], led[LCD_TYPE_5_FDD35]);
		}

		// fdd0
		if ((flag & 0x11000000) == 0x01000000) {
			copy_led(led_pt[LED_POS_FDD0], led[LED_TYPE_VG]);
		} else if ((flag & 0x11000000) == 0x11000000) {
			copy_led(led_pt[LED_POS_FDD0], led[LED_TYPE_VR]);
		}
		if (flag & 0x0100000000) {
			copy_led(led_pt[LCD_POS_FDD0], led[LCD_TYPE_FDD0]);
		}
		// fdd1
		if ((flag & 0x22000000) == 0x02000000) {
			copy_led(led_pt[LED_POS_FDD1], led[LED_TYPE_VG]);
		} else if ((flag & 0x22000000) == 0x22000000) {
			copy_led(led_pt[LED_POS_FDD1], led[LED_TYPE_VR]);
		}
		if (flag & 0x0200000000) {
			copy_led(led_pt[LCD_POS_FDD1], led[LCD_TYPE_FDD1]);
		}
		// fdd2
		if ((flag & 0x44000000) == 0x04000000) {
			copy_led(led_pt[LED_POS_FDD2], led[LED_TYPE_VG]);
		} else if ((flag & 0x44000000) == 0x44000000) {
			copy_led(led_pt[LED_POS_FDD2], led[LED_TYPE_VR]);
		}
		if (flag & 0x0400000000) {
			copy_led(led_pt[LCD_POS_FDD2], led[LCD_TYPE_FDD2]);
		}
		// fdd3
		if ((flag & 0x88000000) == 0x08000000) {
			copy_led(led_pt[LED_POS_FDD3], led[LED_TYPE_VG]);
		} else if ((flag & 0x88000000) == 0x88000000) {
			copy_led(led_pt[LED_POS_FDD3], led[LED_TYPE_VR]);
		}
		if (flag & 0x0800000000) {
			copy_led(led_pt[LCD_POS_FDD3], led[LCD_TYPE_FDD3]);
		}

		// cmt counter
		uint32 count = ((flag >> 8) & 0xffff);
		int i = (count / 12);
		int f = (count % 12);
		POINT pt={0,0};
		pt.y = (i % 10) * led_pt[LED_POS_DIGIT1].sz.cy + f;
		copy_led_s(led_pt[LED_POS_DIGIT1], led[LED_TYPE_DIGIT], pt);
		pt.y = ((i / 10) % 10) * led_pt[LED_POS_DIGIT2].sz.cy + ((i % 10) == 9 ? f : 0);
		copy_led_s(led_pt[LED_POS_DIGIT2], led[LED_TYPE_DIGIT], pt);
		pt.y = ((i / 100) % 10) * led_pt[LED_POS_DIGIT3].sz.cy + ((i % 100) == 99 ? f : 0);
		copy_led_s(led_pt[LED_POS_DIGIT3], led[LED_TYPE_DIGIT], pt);

		if (hLedBox && visible && !inside) {
			InvalidateRect(hLedBox, NULL, TRUE);
			UpdateWindow(hLedBox);
		}
	}

	flag_old = flag;
}

void LedBox::copy_led(led_pt_t &p, led_t &l)
{
	BitBlt(hMainDib, p.pt.x, p.pt.y, l.sz.cx, l.sz.cy, l.hDC, 0, 0, SRCCOPY);
}

void LedBox::copy_led_s(led_pt_t &p, led_t &l, POINT &pt)
{
	BitBlt(hMainDib, p.pt.x, p.pt.y, p.sz.cx, p.sz.cy, l.hDC, pt.x, pt.y, SRCCOPY);
}

void LedBox::SetMode(int val)
{
	val = (val < 0) ? 0 : (val > 1 ? 1 : val);
	dist_set[mode].x = dist.x;
	dist_set[mode].y = dist.y;
	mode = val;
	dist.x = dist_set[mode].x;
	dist.y = dist_set[mode].y;
}
#if 0
void LedBox::SetQuadrant()
{
	if (hLedBox) {
		HWND hWnd = GetParent(hLedBox);
		GetWindowInfo(hWnd, &wi);
		GetWindowInfo(hLedBox, &wid);
	}
}
#endif

void LedBox::SetPos(int left, int top, int right, int bottom, int place)
{
	win_pt.left = left;
	win_pt.top = top;
	win_pt.right = right;
	win_pt.bottom = bottom;
	if (place != -1) win_pt.place = place;

	if (win_pt.place & 1) {
		// base right
		parent_pt.x = right - win_sz.cx;
	} else {
		// base left
		parent_pt.x = left;
	}
	if (win_pt.place & 2) {
		// base bottom
		parent_pt.y = bottom - win_sz.cy;
	} else {
		// base top
		parent_pt.y = top;
	}
}

void LedBox::SetPos(int place)
{
	SetPos(win_pt.left, win_pt.top, win_pt.right, win_pt.bottom, place);
}

void LedBox::Draw(HDC hdc)
{
	if (!visible || !inside || !enable) return;

	// copy to main context
	BitBlt(hdc, parent_pt.x, parent_pt.y, win_sz.cx, win_sz.cy, hMainDib, 0, 0, SRCCOPY);
}

void LedBox::Draw(LPDIRECT3DSURFACE9 suf)
{
	if (!visible || !inside || !enable || !suf) return;

	HDC hdc;
	HRESULT hre = suf->GetDC(&hdc);
	if (hre == D3D_OK) {
		// copy to main context
		BitBlt(hdc, parent_pt.x, parent_pt.y, win_sz.cx, win_sz.cy, hMainDib, 0, 0, SRCCOPY);
	}
	suf->ReleaseDC(hdc);
}
