/** @file parseopt.cpp

	Skelton for retropc emulator

	@author Sasaji
	@date   2015.05.18 -

	@brief [ parse options ]
*/

#include "parseopt.h"
#include <tchar.h>
#include "config.h"
#include "vm/vm.h"
#include "utility.h"

static _TCHAR app_path[_MAX_PATH];
static _TCHAR app_name[_MAX_PATH];
static _TCHAR ini_path[_MAX_PATH];
static _TCHAR ini_file[_MAX_PATH];

static _TCHAR *tape_file;
static _TCHAR *disk_file[USE_DRIVE];
static _TCHAR *state_file;
static _TCHAR *autokey_file;
static _TCHAR *reckey_file;

const _TCHAR *get_app_path()
{
	return app_path;
}

const _TCHAR *get_app_name()
{
	return app_name;
}

const _TCHAR *get_ini_path()
{
	return ini_path;
}

const _TCHAR *get_ini_file()
{
	return ini_file;
}

/**
 *	get and parse options
 */
int get_options(LPTSTR szCmdLine)
{
	_TCHAR *optarg[10];
	int     optlen[10];
	int     optargc = 0;

	_TCHAR path[_MAX_PATH];
	memset(path, 0, sizeof(path));
	memset(app_path, 0, sizeof(app_path));
	memset(ini_path, 0, sizeof(ini_path));
	memset(ini_file, 0, sizeof(ini_file));

	// allocate buffer
	tape_file = new _TCHAR[_MAX_PATH];
	memset(tape_file, 0, sizeof(_TCHAR) * _MAX_PATH);
	for(int i=0; i<USE_DRIVE; i++) {
		disk_file[i] = new _TCHAR[_MAX_PATH];
		memset(disk_file[i], 0, sizeof(_TCHAR) * _MAX_PATH);
	}
	state_file = new _TCHAR[_MAX_PATH];
	memset(state_file, 0, sizeof(_TCHAR) * _MAX_PATH);
	autokey_file = new _TCHAR[_MAX_PATH];
	memset(autokey_file, 0, sizeof(_TCHAR) * _MAX_PATH);
	reckey_file = new _TCHAR[_MAX_PATH];
	memset(reckey_file, 0, sizeof(_TCHAR) * _MAX_PATH);

	// set application path and name
	GetModuleFileName(NULL, app_path, _MAX_PATH);
	UTILITY::get_dir_and_basename(app_path, app_name);

	// split options
	bool quoting = false;
	_TCHAR *p = szCmdLine;
	_TCHAR *ps = szCmdLine;
	if (*p != _T('\0')) {
		for(; ; p++) {
			if (*p == _T('"')) {
				quoting = !quoting;
				continue;
			}
			if ((!quoting && *p == _T(' ')) || *p == _T('\0')) {
				if (p > ps) {
					optarg[optargc] = ps;
					optlen[optargc] = (int)(p - ps);
					if (optarg[optargc][0] == _T('"')) {
						optarg[optargc]++;
						optlen[optargc]--;
					}
					if (optarg[optargc][optlen[optargc]-1] == _T('"')) {
						optlen[optargc]--;
					}
					if (optlen[optargc] > 0) {
						optargc++;
					}
				}
				if (optargc >= 10 || *p == _T('\0')) break;
				ps = p + 1;
			}
		}
	}
	// parse options
	_TCHAR opt = _T('\0');
	int    optind = 0;
	bool finished = false;
	for(optind = 0; optind < optargc; optind++) {
		if (_tcschr(_T("-/"), optarg[optind][0]) != NULL) {
			opt = optarg[optind][1];
			if (_tcschr(_T("itdsak"), opt) != NULL) {
				continue;
			}
			if (_tcschr(_T("hH"), opt) == NULL) {
				opt = _T('?');
			}
		}
		///
		switch(opt) {
			case _T('i'):
				memset(ini_file, 0, sizeof(ini_file));
				_tcsncpy(ini_file, optarg[optind], optlen[optind] < _MAX_PATH ? optlen[optind] : _MAX_PATH - 1);
				break;
			case _T('d'):
				for(int drv=0; drv<USE_DRIVE; drv++) {
					if (disk_file[drv][0] == _T('\0')) {
						memset(disk_file[drv], 0, sizeof(_TCHAR) * _MAX_PATH);
						_tcsncpy(disk_file[drv], optarg[optind], optlen[optind] < _MAX_PATH ? optlen[optind] : _MAX_PATH - 1);
						break;
					}
				}
				break;
			case _T('t'):
				memset(tape_file, 0, sizeof(_TCHAR) * _MAX_PATH);
				_tcsncpy(tape_file, optarg[optind], optlen[optind] < _MAX_PATH ? optlen[optind] : _MAX_PATH - 1);
				break;
			case _T('s'):
				memset(state_file, 0, sizeof(_TCHAR) * _MAX_PATH);
				_tcsncpy(state_file, optarg[optind], optlen[optind] < _MAX_PATH ? optlen[optind] : _MAX_PATH - 1);
				break;
			case _T('a'):
				memset(autokey_file, 0, sizeof(_TCHAR) * _MAX_PATH);
				_tcsncpy(autokey_file, optarg[optind], optlen[optind] < _MAX_PATH ? optlen[optind] : _MAX_PATH - 1);
				break;
			case _T('k'):
				memset(reckey_file, 0, sizeof(_TCHAR) * _MAX_PATH);
				_tcsncpy(reckey_file, optarg[optind], optlen[optind] < _MAX_PATH ? optlen[optind] : _MAX_PATH - 1);
				break;
			case _T('h'):
			case _T('H'):
			case _T('?'):
				_tprintf(_T("Usage: %s [-h] [-i <ini_file>] [-t <tape_file>] [-d <disk_file>] [-s <state_file>] [-a <autokey_file>] [-k <recordkey_file>] [<support_file> ...]\n"), app_name);
				exit(0);
				break;
			default:
				finished = true;
				break;
		}
		opt = _T('\0');
		if (finished) break;
	}
	for(; optind < optargc && optarg[optind][0] != _T('\0'); optind++) {
		memset(path, 0, sizeof(path));
		_tcsncpy(path, optarg[optind], optlen[optind] < _MAX_PATH ? optlen[optind] : _MAX_PATH - 1);
		int rc = GUI::CheckSupportedFile(path);
		switch(rc) {
		case 1:
			memset(tape_file, 0, sizeof(_TCHAR) * _MAX_PATH);
			_tcscpy(tape_file, path);
			break;
		case 2:
			for(int drv=0; drv<USE_DRIVE; drv++) {
				if (disk_file[drv][0] == _T('\0')) {
					memset(disk_file[drv], 0, sizeof(_TCHAR) * _MAX_PATH);
					_tcscpy(disk_file[drv], path);
					break;
				}
			}
			break;
		case 3:
			memset(state_file, 0, sizeof(_TCHAR) * _MAX_PATH);
			_tcscpy(state_file, path);
			break;
		case 4:
			memset(autokey_file, 0, sizeof(_TCHAR) * _MAX_PATH);
			_tcscpy(autokey_file, path);
			break;
		case 5:
			memset(ini_file, 0, sizeof(ini_file));
			_tcscpy(ini_file, path);
			break;
		case 6:
			memset(reckey_file, 0, sizeof(_TCHAR) * _MAX_PATH);
			_tcscpy(reckey_file, path);
			break;
		default:
			break;
		}
	}

	// check ini file
	if (ini_file[0] != _T('\0')) {
		if (UTILITY::check_file_extension(ini_file, _T("ini"))) {
			_TCHAR ini_name[_MAX_PATH];
			UTILITY::get_dir_and_basename(ini_file, ini_path, ini_name);
		}
		UTILITY::add_path_separator(ini_path);
	} else {
		_tcsncpy(ini_path, app_path, _MAX_PATH);
		_tcsncpy(ini_file, ini_path, _MAX_PATH);
		_tcsncat(ini_file, _T(CONFIG_NAME), _MAX_PATH);
		_tcsncat(ini_file, _T(".ini"), _MAX_PATH);
	}

	return 0;
}

/**
 *	open recent file
 */
void open_recent_file(GUI *gui)
{
	_TCHAR path[_MAX_PATH];

	if (gui) {
		if (tape_file[0] != '\0') {
			gui->PostEtLoadDataRecMessage(tape_file);
		}
#ifdef USE_FD1
		// auto open recent file
		for(int drv=0; drv<USE_DRIVE; drv++) {
			path[0] = _T('\0');
			int bank_num = 0;
			if (disk_file[drv][0] != _T('\0') && _tcscmp(path, disk_file[drv]) != 0) {
				// specified file in the option
				path[0] = _T('\0');
				_tcscpy(path, disk_file[drv]);
			} else if ((config.mount_fdd & (1 << drv)) != 0 && _tcslen(config.recent_disk_path[drv][0].path) > 0) {
				// recent file
				path[0] = _T('\0');
				_tcscpy(path, config.recent_disk_path[drv][0].path);
				bank_num = config.recent_disk_path[drv][0].num;
			}
			if (path[0] != _T('\0')) {
				gui->PostEtOpenFloppyMessage(drv, path, bank_num, 0, false);
			}
		}
#endif
		if (state_file[0] != _T('\0')) {
			gui->PostEtLoadStatusMessage(state_file);
		}
		if (autokey_file[0] != _T('\0')) {
			gui->PostEtLoadAutoKeyMessage(autokey_file);
		}
		if (reckey_file[0] != _T('\0')) {
			gui->PostEtLoadRecKeyMessage(reckey_file);
		}
	}

	// release buffer
	delete[] tape_file; tape_file = NULL;
	for(int i=0; i<USE_DRIVE; i++) {
		delete[] disk_file[i]; disk_file[i] = NULL;
	}
	delete[] state_file; state_file = NULL;
	delete[] autokey_file; autokey_file = NULL;
	delete[] reckey_file; reckey_file = NULL;
}
