/** @file rec_video.h

	Skelton for retropc emulator

	@author Sasaji
	@date   2015.05.18 -

	@brief [ record video ]
*/

#ifndef _RECORD_VIDEO_H_
#define _RECORD_VIDEO_H_

#include "../common.h"
#include "../dibsection.h"
#ifdef USE_REC_VIDEO_VFW
class VFW_REC_VIDEO;
#endif
#ifdef USE_REC_VIDEO_FFMPEG
class FFM_REC_VIDEO;
#endif

enum en_record_video_type {
	RECORD_VIDEO_TYPE_VFW = 1,
	RECORD_VIDEO_TYPE_QTKIT,
	RECORD_VIDEO_TYPE_FFMPEG,
	CAPTURE_SCREEN_TYPE_BMP = 11,
	RECORD_VIDEO_TYPE_UNKNOWN
};

class EMU;

class REC_VIDEO
{
private:
	EMU *emu;
	bool now_recording;
	int rec_type;
	int rec_fps;
	int rec_count;
	int rec_denomi;
	_TCHAR rec_path[_MAX_PATH];

	DibSection *dibRecVideo;

#ifdef USE_REC_VIDEO_VFW
	VFW_REC_VIDEO *vfwvideo;
#endif
#ifdef USE_REC_VIDEO_FFMPEG
	FFM_REC_VIDEO *ffmvideo;
#endif
	void CreateSurface(int w, int h);

public:
	REC_VIDEO(EMU *new_emu);
	~REC_VIDEO();

	/// true:OK false:ERROR
	bool Start(int type, int fps, VmRectWH *srcrect, DibSection *srcsurface, bool show_dialog);
	void Stop();
	bool Restart();
	bool Record(VmRectWH *srcrect, DibSection *srcsurface);
	bool IsRecordFrame();
	bool NowRecording() {
		return now_recording;
	}
	bool Capture(int type, VmRectWH *srcrect, DibSection *srcsurface);
	void CreateFileName(_TCHAR *file_path);

	bool IsEnabled(int type);
	const _TCHAR **GetCodecList(int type);
	const _TCHAR **GetQualityList(int type);

};

#endif /* _RECORD_VIDEO_H_ */
