/** @file keyboard_bind.h

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2012.04.08 -

	@brief [ keyboard bind ]
*/

#include "../vm.h"

#define VK_KEY_TYPE "WIN"

/* l3        -> win      */
/* [caps]    -> capslock */
/* [graph]   -> muhenkan */
/* [kana]    -> katakana */
/* [break]   -> F9       */
/* [num ?]   -> end      */
/* [del  ]   -> bs       */
/* [num del] -> del      */
/* reset     -> F11      */
/* (pause)   -> pause    */
/* (alt)     -> alt      */

static const uint32 scan2key_defmap[KEYBIND_KEYS][KEYBIND_ASSIGN]={
//	 space       up          num?        left        down        right       ctrl        shift
//   0x80,       0x81,       0x82,       0x83,       0x84,       0x85,       0x86,       0x87,
	{0x20,0x00},{0x26,0x00},{0x23,0x00},{0x25,0x00},{0x28,0x00},{0x27,0x00},{0x11,0x00},{0x10,0x00},
//	             caps        kana        graph       esc         num8        num9        num*
//   0x88,       0x89,       0x8a,       0x8b,       0x8c,       0x8d,       0x8e,       0x8f,
	{0x00,0x00},{0x14,0x00},{0x15,0x00},{0x1d,0x00},{0x1b,0x00},{0x68,0x00},{0x69,0x00},{0x6a,0x00},
//	 7           4           6           8           0           ^           =-          3
//   0x90,       0x91,       0x92,       0x93,       0x94,       0x95,       0x96,       0x97
	{0x37,0x00},{0x34,0x00},{0x36,0x00},{0x38,0x00},{0x30,0x00},{0xde,0x00},{0xbd,0x00},{0x33,0x00},
//	 num ins/del 5           1           2           9           num7        ins/del     \
//   0x98,       0x99,       0x9a,       0x9b,       0x9c,       0x9d,       0x9e,       0x9f,
	{0x2e,0x00},{0x35,0x00},{0x31,0x00},{0x32,0x00},{0x39,0x00},{0x67,0x00},{0x08,0x00},{0xdc,0x00},
//	 U           R           Y           I           P           [           @           num0
//   0xa0,       0xa1,       0xa2,       0xa3,       0xa4,       0xa5,       0xa6,       0xa7,
	{0x55,0x00},{0x52,0x00},{0x59,0x00},{0x49,0x00},{0x50,0x00},{0xdb,0x00},{0xc0,0x00},{0x60,0x00},
//	 Q           T           W           E           O           num.        home/cls    return
//   0xa8,       0xa9,       0xaa,       0xab,       0xac,       0xad,       0xae,       0xaf,
	{0x51,0x00},{0x54,0x00},{0x57,0x00},{0x45,0x00},{0x4f,0x00},{0x6e,0x00},{0x24,0x00},{0x0d,0x00},
//   J           F           H           K           +;          ]           *:          num4
//   0xb0,       0xb1,       0xb2,       0xb3,       0xb4,       0xb5,       0xb6,       0xb7,
	{0x4a,0x00},{0x46,0x00},{0x48,0x00},{0x4b,0x00},{0xbb,0x00},{0xdd,0x00},{0xba,0x00},{0x64,0x00},
//   A           G           S           D           L           num5        num6        num-
//   0xb8,       0xb9,       0xba,       0xbb,       0xbc,       0xbd,       0xbe,       0xbf,
	{0x41,0x00},{0x47,0x00},{0x53,0x00},{0x44,0x00},{0x4c,0x00},{0x65,0x00},{0x66,0x00},{0x6d,0x00},
//   M           V           N           <,          ?/          num/        _           num1
//   0xc0,       0xc1,       0xc2,       0xc3,       0xc4,       0xc5,       0xc6,       0xc7,
	{0x4d,0x00},{0x56,0x00},{0x4e,0x00},{0xbc,0x00},{0xbf,0x00},{0x6f,0x00},{0xe2,0x00},{0x61,0x00},
//   Z           B           X           C           >.          num2        num3        num+
//   0xc8,       0xc9,       0xca,       0xcb,       0xcc,       0xcd,       0xce,       0xcf,
	{0x5a,0x00},{0x42,0x00},{0x58,0x00},{0x43,0x00},{0xbe,0x00},{0x62,0x00},{0x63,0x00},{0x6b,0x00},
//   F1          F2          F3          F4          F5
//   0xd0,       0xd1,       0xd2,       0xd3,       0xd4,       0xd5,       0xd6,       0xd7,
	{0x70,0x00},{0x71,0x00},{0x72,0x00},{0x73,0x00},{0x74,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},

	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//   (pause)     (alt)
	{0x13,0x00},{0x12,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//                                                                                       reset
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x7a,0x00},
//   break      (property)
	{0x78,0x00},{0x00,0x00}
};

static const uint32 scan2joy_defmap[KEYBIND_KEYS][KEYBIND_ASSIGN]={
//	 space       up          num?        left        down        right       ctrl        shift
//   0x80,       0x81,       0x82,       0x83,       0x84,       0x85,       0x86,       0x87,
	{0x10,0x00},{0x01,0x00},{0x00,0x00},{0x04,0x00},{0x02,0x00},{0x08,0x00},{0x00,0x00},{0x00,0x00},
//	             caps        kana        graph       esc         num8        num9        num*
//   0x88,       0x89,       0x8a,       0x8b,       0x8c,       0x8d,       0x8e,       0x8f,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x01},{0x00,0x09},{0x00,0x00},
//	 7           4           6           8           0           ^           =-          3
//   0x90,       0x91,       0x92,       0x93,       0x94,       0x95,       0x96,       0x97
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//	 num ins/del 5           1           2           9           num7        ins/del     \
//   0x98,       0x99,       0x9a,       0x9b,       0x9c,       0x9d,       0x9e,       0x9f,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x05},{0x00,0x00},{0x00,0x00},
//	 U           R           Y           I           P           [           @           num0
//   0xa0,       0xa1,       0xa2,       0xa3,       0xa4,       0xa5,       0xa6,       0xa7,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x10},
//	 Q           T           W           E           O           num.        home/cls    return
//   0xa8,       0xa9,       0xaa,       0xab,       0xac,       0xad,       0xae,       0xaf,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//   J           F           H           K           +;          ]           *:          num4
//   0xb0,       0xb1,       0xb2,       0xb3,       0xb4,       0xb5,       0xb6,       0xb7,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x04},
//   A           G           S           D           L           num5        num6        num-
//   0xb8,       0xb9,       0xba,       0xbb,       0xbc,       0xbd,       0xbe,       0xbf,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x08},{0x00,0x00},
//   M           V           N           <,          ?/          num/        _           num1
//   0xc0,       0xc1,       0xc2,       0xc3,       0xc4,       0xc5,       0xc6,       0xc7,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x06},
//   Z           B           X           C           >.          num2        num3        num+
//   0xc8,       0xc9,       0xca,       0xcb,       0xcc,       0xcd,       0xce,       0xcf,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x02},{0x00,0x0a},{0x00,0x00},
//   F1          F2          F3          F4          F5
//   0xd0,       0xd1,       0xd2,       0xd3,       0xd4,       0xd5,       0xd6,       0xd7,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},

	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//   (pause)
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//                                                                                       reset
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//   break      (property)
	{0x00,0x00},{0x00,0x00}
};

/// for keybind dialog box
static const uint16 kb2scan_map[128] = {
//  0    1    2    3    4    5    6    7    8    9    a    b    c    d    e    f
	0xff,0x80,0x7f,0xff,0x60,0x61,0x1e,0x18,0xff,0xff,0xff,0xff,0x2e,0x2f,0xff,0xff,
	0x07,0x06,0xff,0xff,0x09,0x0a,0x0b,0xff,0xff,0xff,0xff,0x0c,0x05,0x03,0x01,0x04,
	0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0x14,0x1a,0x1b,0x17,0x11,0x19,0x12,0x10,0x13,0x1c,0x36,0x34,0x43,0x16,0x4c,0x44,
	0x26,0x38,0x49,0x4b,0x3b,0x2b,0x31,0x39,0x32,0x23,0x30,0x33,0x3c,0x40,0x42,0x2c,
	0x24,0x28,0x21,0x3a,0x29,0x20,0x41,0x2a,0x4a,0x22,0x48,0x25,0x1f,0x35,0x15,0x46,
	0x27,0x47,0x4d,0x4e,0x37,0x3d,0x3e,0x1d,0x0d,0x0e,0x0f,0x4f,0x02,0x3f,0x2d,0x45,
	0x50,0x51,0x52,0x53,0x54,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
};

