/** @file mb8866.h

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@par Origin
	mb8877.cpp
	@author Sasaji
	@date   2012.03.24 -

	@brief [ MB8866 modoki (same as MB8877/MB8876) ]
*/

#ifndef _MB8866_H_
#define _MB8866_H_

#include "../vm.h"
#include "../../emu.h"
#include "../device.h"

#define SIG_MB8866_DRIVEREG	0
#define SIG_MB8866_SIDEREG	1
#define SIG_MB8866_MOTOR	2

// #define _DEBUG_MB8866

/**
	@brief MB8866 modoki (same as MB8877/MB8876) - Floppy Disk Controller
*/
class MB8866 : public DEVICE
{
private:
	FLOPPY *d_fdd;

	// config
	bool ignore_crc;

	// output signals
	outputs_t outputs_irq;
	outputs_t outputs_drq;
#ifdef _DEBUG_MB8866
	DEVICE *d_cpu;
#endif


	// registor
	uint8 status;
	uint8 cmdreg;
	uint8 trkreg;
	uint8 secreg;
	uint8 datareg;
	uint8 cmdtype;

	// event
	int register_id[7];

	// status
	bool now_search;
	bool now_seek, after_seek;
	int seektrk;
	bool seekvct;
	bool now_irq;
#ifdef _DEBUG_MB8866
	uint32 val0_prev;
#endif

	int data_idx;

	//for resume
#pragma pack(1)
	struct vm_state_st {
		int register_id[7];

		uint8 status;
		uint8 cmdreg;
		uint8 trkreg;
		uint8 secreg;

		uint8 datareg;
		uint8 cmdtype;
		int seektrk;
		int data_idx;
		uint8 flags;

		char reserved[5];
	};
#pragma pack()

	void cancel_my_events();

	// image handler
	uint8 search_track();
	uint8 search_sector(int side, int sct, bool compare);
	uint8 search_addr();
	bool make_track();
	bool parse_track();

	// command
	void process_cmd();
	void cmd_restore();
	void cmd_seek();
	void cmd_step();
	void cmd_stepin();
	void cmd_stepout();
	void cmd_readdata();
	void cmd_writedata();
	void cmd_readaddr();
	void cmd_readtrack();
	void cmd_writetrack();
	void cmd_forceint();

	// irq/dma
	inline void set_irq(bool val);
	inline void set_drq(bool val);

public:
	MB8866(VM* parent_vm, EMU* parent_emu, const char* identifier) : DEVICE(parent_vm, parent_emu, identifier) {
		set_class_name("MB8866");
		init_output_signals(&outputs_irq);
		init_output_signals(&outputs_drq);
		d_fdd = NULL;
#ifdef _DEBUG_MB8866
		d_cpu = NULL;
#endif
	}
	~MB8866() {}

	// common functions
	void initialize();
	void release();
	void reset();
	void write_io8(uint32 addr, uint32 data);
	uint32 read_io8(uint32 addr);
	void write_dma_io8(uint32 addr, uint32 data);
	uint32 read_dma_io8(uint32 addr);
	void write_signal(int id, uint32 data, uint32 mask);
	uint32 read_signal(int ch);
	void event_callback(int event_id, int err);
	void update_config();

	// unique function
	void set_context_irq(DEVICE* device, int id, uint32 mask) {
		register_output_signal(&outputs_irq, device, id, mask);
	}
	void set_context_drq(DEVICE* device, int id, uint32 mask) {
		register_output_signal(&outputs_drq, device, id, mask);
	}
	void set_context_fdd(FLOPPY* device) {
		d_fdd = device;
	}
#ifdef _DEBUG_MB8866
	void set_context_cpu(DEVICE* device) {
		d_cpu = device;
	}
#endif
	void save_state(FILEIO *fp);
	void load_state(FILEIO *fp);
};

#endif
